@@ -1,15 +1,12 @@
 # -*- coding: utf-8 -*-
 import logging
 import os
-# from crosspm.helpers.package import Package
-from crosspm.helpers.exceptions import *
-from datetime import datetime, timedelta
 import time
+from crosspm.helpers.package import md5sum
+from datetime import datetime, timedelta
 
 
-class Cache(object):
-    _clear = {}
-
+class Cache:
     def __init__(self, config, cache_data):
         self._log = logging.getLogger('crosspm')
         self._config = config
@@ -17,8 +14,10 @@ class Cache(object):
         if not os.path.exists(self._cache_path):
             os.makedirs(self._cache_path)
 
-        self.packed_path = os.path.realpath(os.path.join(self._cache_path, 'archive'))
-        self.unpacked_path = os.path.realpath(os.path.join(self._cache_path, 'cache'))
+        self.path = {
+            'packed': os.path.realpath(os.path.join(self._cache_path, 'archive')),
+            'unpacked': os.path.realpath(os.path.join(self._cache_path, 'cache')),
+        }
         self.temp_path = os.path.realpath(os.path.join(self._cache_path, 'tmp'))
 
         self._clear = cache_data.get('clear', {})
@@ -28,14 +27,14 @@ class Cache(object):
                 self._clear['auto'] = False
         try:
             self._clear['auto'] = bool(self._clear['auto'])
-        except:
+        except Exception:
             self._clear['auto'] = False
 
         self._clear['size'] = self.str_to_size(self._clear.get('size', '0'))
 
         try:
             self._clear['days'] = int(self._clear.get('days', '0'))
-        except:
+        except Exception:
             self._clear['days'] = 0
 
         _unique = config.get_fails('unique', None)
@@ -60,7 +59,7 @@ class Cache(object):
         _size = str(size).replace(' ', '').lower()
         try:
             _size0 = float(_size[:-2])
-        except:
+        except Exception:
             _size0 = 0
         _measure = _size[-2:]
         if _measure == 'kb':
@@ -74,14 +73,14 @@ class Cache(object):
                 _size = _size[:-1]
             try:
                 _size = float(_size)
-            except:
+            except Exception:
                 _size = 0
         return _size
 
     def size_to_str(self, size, dec=0):
         try:
             _size = float(size)
-        except:
+        except Exception:
             _size = 0
         _measure = 'b '
         _size0 = round(_size / 1024, 3)
@@ -132,8 +131,8 @@ class Cache(object):
             return res
 
         total = {
-            'packed': get_dir(self.packed_path),
-            'unpacked': get_dir(self.unpacked_path),
+            'packed': get_dir(self.path['packed']),
+            'unpacked': get_dir(self.path['unpacked']),
             'temp': get_dir(self.temp_path),
             'other': get_dir(self._cache_path, False),
         }
@@ -143,7 +142,7 @@ class Cache(object):
         return item['time']
 
     def info(self):
-        print_stdout('Cache info:')
+        print("Cache info:")
 
     def _delete_dir(self, _dirs, max_time=None, del_size=None):
         cleared = [0]
@@ -283,42 +282,44 @@ class Cache(object):
         self._log.info('    oldest: {}'.format(datetime.fromtimestamp(oldest)))
 
     def path_packed(self, package=None, params=None):
-        if params:
-            res = self._storage['packed'].format(**params)
-        elif package:
-            res = self._storage['packed'].format(**(package.get_params(merged=True)))
-        else:
-            res = ''
-        res = os.path.realpath(os.path.join(self.packed_path, res))
-        return res
+        return self.path_any('packed', package, params)
 
     def path_unpacked(self, package=None, params=None):
+        return self.path_any('unpacked', package, params)
+
+    def path_any(self, name, package=None, params=None):
         if params:
-            res = self._storage['unpacked'].format(**params)
+            tmp_params = {}
+            for k, v in params.items():
+                if isinstance(v, list):
+                    v = ".".join([x for x in v if x is not None and x != ''])
+                tmp_params[k] = v
+            res = self._storage[name].format(**tmp_params)
         elif package:
-            res = self._storage['unpacked'].format(**(package.get_params(merged=True)))
+            res = self._storage[name].format(**(package.get_params(merged=True)))
         else:
             res = ''
-        res = os.path.realpath(os.path.join(self.unpacked_path, res))
+        res = os.path.realpath(os.path.join(self.path[name], res))
         return res
 
-    def exists_packed(self, package=None, params=None, pkg_path=''):
+    def exists_packed(self, package=None, params=None, pkg_path='', check_stat=True):
         # Check if file exists and size and time match
         path = self.path_packed(package, params) if not pkg_path else pkg_path
         res = os.path.exists(path)
-        if res and package:
-            _stat_attr = {'ctime': 'st_atime',
-                          'mtime': 'st_mtime',
-                          'size': 'st_size'}
-            _stat_file = os.stat(path)
-            if any(package.stat[k] != getattr(_stat_file, v, -999) for k, v in _stat_attr.items()):
+        if res and package and check_stat:
+            md5file = md5sum(filename=path)
+            if package.md5 != md5file:
                 res = False
-
+                os.remove(path)
         return res, path
 
     def exists_unpacked(self, package, filename=None, params=None, pkg_path=''):
         # TODO: Check if file exists and size and time match
         path = self.path_unpacked(package, params) if not pkg_path else pkg_path
-        res = os.path.exists(path)
+        if os.path.isdir(path):
+            # check that dir is not empty
+            res = bool(os.listdir(path))
+        else:
+            res = os.path.exists(path)
 
         return res, path
