@@ -1,30 +1,55 @@
 # -*- coding: utf-8 -*-
-import re
+import copy
 import fnmatch
+import itertools
+import logging
 import os
+import re
+
+from crosspm.helpers.content import DependenciesContent
 from crosspm.helpers.exceptions import *
 
 
-class Parser(object):
-    # _name = ''
-    # _rules = {}
-    # _config = None
-    _rules_vars = {}
-    _rules_vars_extra = {}
-    _columns = {}
-    _col_types = []
-    _sort = []
-    _index = -1
-
+class Parser:
     def __init__(self, name, data, config):
+        self._rules_vars = {}
+        self._rules_vars_extra = {}
+        self._columns = {}
+        self._defaults = {}
+        self._defaults_masked = {}
+        self._col_types = []
         self._name = name
-        self._sort = data.get('sort', self._sort)
-        self._index = data.get('index', self._index)
-        self._rules = {k: v for k, v in data.items() if k not in ['columns', 'index', 'sort']}
+        self._sort = data.get('sort', [])
+        self._index = data.get('index', -1)
+
+        # Должно быть вида key: str_value
+        self._rules = {k: v for k, v in data.items() if k not in ['columns', 'index', 'sort', 'defaults', 'usedby']}
+
         if 'columns' in data:
             self._columns = {k: self.parse_value_template(v) for k, v in data['columns'].items() if v != ''}
         self._config = config
         self.init_rules_vars()
+        if 'defaults' in data:
+            self.init_defaults(data['defaults'])
+        self._usedby = data.get('usedby', None)
+
+    def get_usedby_aql(self, params):
+        """
+        Возвращает запрос AQL (без репозитория), из файла конфигурации
+        :param params:
+        :return:
+        """
+        if self._usedby is None:
+            return None
+
+        _result = {}
+        params = self.merge_valued(params)
+        for k, v in self._usedby['AQL'].items():
+            if isinstance(v, str):
+                k = k.format(**params)
+                v = v.format(**params)
+            _result[k] = v
+        return _result
 
     def get_vars(self):
         _vars = []
@@ -33,6 +58,29 @@ class Parser(object):
             # _vars += [x for x in _rule_vars if x not in _vars]
         return _vars
 
+    def init_defaults(self, defaults):
+        if isinstance(defaults, dict):
+            self._defaults = {k: v for k, v in defaults.items()}
+            self._defaults_masked = {}
+            for _rules_name, _rules in self._rules.items():
+                if _rules_name == 'path':
+                    continue
+                self._defaults_masked[_rules_name] = []
+                for _rule in _rules:
+                    _rule_tmp = _rule
+                    for _name in [x for x in re.findall('{.*?}', _rule_tmp)]:
+                        _name_replace = _name[1:-1].split('|')[0]
+                        _rule_tmp.replace(_name, _name_replace)
+                        if _name_replace not in self._defaults:
+                            self._defaults[_name_replace] = ''
+                    _tmp = [x.strip() for x in _rule_tmp.split('=')]
+                    _mask = re.sub('{.*?}', '*', _tmp[0])
+                    _val = ''
+                    _key = _tmp[0].format(**self._defaults)
+                    if len(_tmp) > 1:
+                        _val = _tmp[1].format(**self._defaults)
+                    self._defaults_masked[_rules_name].append({'mask': _mask, 'key': _key, 'value': _val})
+
     def init_rules_vars(self):
         self._rules_vars = {}
         for _name in self._rules:
@@ -65,8 +113,11 @@ class Parser(object):
                         #     _rules = _rules[0]
 
     def parse_by_mask(self, column, value, types=False, ext_mask=False):
+        # см https://habrahabr.ru/post/269759/
         if column not in self._columns:
             return value  # nothing to parse
+        if isinstance(value, list):
+            return value[:]
         _res = []
         # extMask = extMask and value == '*'
         orig_value = value
@@ -147,11 +198,13 @@ class Parser(object):
 
     def merge_with_mask(self, column, value):
         if column not in self._columns:
-            if type(value) in [list, tuple]:
+            if isinstance(value, (list, tuple)):
                 # TODO: Check for value is not None - if it is, raise "column value not set"
+                # if None in value:
+                #     value = ['' if x is None else x for x in value]
                 value = ''.join(value)
             return value  # nothing to parse
-        if type(value) not in [list, tuple]:
+        if not isinstance(value, (list, tuple)):
             return value  # nothing to parse
         _res = ''
         _res_tmp = ''
@@ -201,29 +254,43 @@ class Parser(object):
         if column not in self._columns:
             _res = True  # nothing to validate
             _res_value = value
-        elif type(param) not in [list, tuple]:
+        elif not isinstance(param, (list, tuple)):
             _res = False
         else:
             _res = True
             for i, (_tmp, tp) in enumerate(self.parse_by_mask(column, value, True)):
-                if tp == 'int':
-                    try:
-                        _tmp = int(_tmp)
-                    except:
-                        _tmp = str(_tmp)
-                if not self.validate_atom(_tmp, param[i]):
-                    _res = False
-                    _res_value = []
-                    break
-                else:
-                    _res_value.append(_tmp)
+                _is_empty_optional = (len(_tmp) == 0 and param[i] is None)
+                if tp == 'int' and not _is_empty_optional:
+                    if re.match(r"^[0-9]+$", _tmp):
+                        if not (len(_tmp) > 1 and _tmp.startswith('0')):
+                            _tmp = int(_tmp)
+                        else:
+                            _tmp = str(_tmp)
+                        if self.validate_atom(_tmp, param[i]):
+                            _res_value.append(_tmp)
+                        else:
+                            _res = False
+                            _res_value = []
+                            break
+                    else:
+                        _res = False
+                        _res_value = []
+                        break
+                elif tp == 'str' or _is_empty_optional:
+                    _tmp = str(_tmp)
+                    if self.validate_atom(_tmp, param[i]):
+                        _res_value.append(_tmp)
+                    else:
+                        _res = False
+                        _res_value = []
+                        break
 
         return _res, _res_value
 
     @staticmethod
     def validate_atom(value, text):
         _sign = ''
-        if text:
+        if text is not None:
             if text.startswith(('>=', '<=', '==',)):
                 _sign = text[:2]
                 text = text[2:]
@@ -234,13 +301,13 @@ class Parser(object):
                     _sign = '=='
 
         var1 = value
-        var2 = text if text else '*'
-        if type(var1) is int:
+        var2 = text if text is not None else '*'
+        if isinstance(var1, int):
             try:
                 var2 = int(var2)
                 if not _sign:
                     _sign = '=='
-            except:
+            except ValueError:
                 var1 = str(var1)
 
         if _sign:
@@ -258,7 +325,7 @@ class Parser(object):
                 _res0 = [_value0]
                 if _col_name in self._rules_vars_extra[_rule_name][rule_number]:
                     _res0 += self._rules_vars_extra[_rule_name][rule_number][_col_name]
-                for _res1 in _res0:
+                for _res1 in sorted(_res0, key=lambda x: 0 - len(x)):
                     yield _res1
 
             def get_symbol_in_mask(_sym1, _val_mask):
@@ -325,6 +392,7 @@ class Parser(object):
             rule = self._rules[_rule_name][rule_number]
             rule_parsed = self.parse_value_template(rule)
             _res_params = {}
+            _res_params_raw = {}
             for x, _part in enumerate(rule_parsed):
                 for y, _subpart in enumerate(_part[0]):
                     if _subpart[1]:
@@ -339,6 +407,7 @@ class Parser(object):
                                     _valid, _valid_value = self.validate_by_mask(_subpart[0], _atom, _value_item)
                                     if _valid:
                                         _res_params[_subpart[0]] = _valid_value
+                                        _res_params_raw[_subpart[0]] = _atom
                                         _new_path += _atom
                                         _path = _atom_item['path']
                                         _match = True
@@ -346,11 +415,12 @@ class Parser(object):
                                 if _match:
                                     break
                             if not _match:
-                                return False, {}
+                                return False, {}, {}
                         else:
                             if _value is None:
                                 _match = False
-                                for _value_item in self._config.get_values(_subpart[0]):
+                                for _value_item in sorted(self._config.get_values(_subpart[0]),
+                                                          key=lambda x: 0 - len(x)):
                                     _atom = _path[:len(_value_item)]
                                     if fnmatch.fnmatch(_atom, _value_item):  # may be just comparing would be better
                                         _res_params[_subpart[0]] = _atom
@@ -359,7 +429,7 @@ class Parser(object):
                                         _match = True
                                         break
                                 if not _match:
-                                    return False, {}
+                                    return False, {}, {}
                             else:
                                 # it's a plain value
                                 _plain = not any(x in _value for x in ('>=', '<=', '==', '>', '<', '=', '*'))
@@ -383,14 +453,15 @@ class Parser(object):
                                     else:
                                         for _value_item in iter_with_extras(_subpart[0], _value):
                                             _atom = _path[:len(_value_item)]
-                                            if fnmatch.fnmatch(_atom, _value_item):  # may be just comparing would be better
+                                            if fnmatch.fnmatch(_atom,
+                                                               _value_item):  # may be just comparing would be better
                                                 _res_params[_subpart[0]] = _atom
                                                 _new_path += _atom
                                                 _path = _path[len(_value_item):]
                                                 _match = True
                                                 break
                                     if not _match:
-                                        return False, {}
+                                        return False, {}, {}
                                 else:
                                     _atoms = get_atom(x, y, _path, self._columns[_subpart[0]])
                                     _match = False
@@ -406,7 +477,7 @@ class Parser(object):
                                         if _match:
                                             break
                                     if not _match:
-                                        return False, {}
+                                        return False, {}, {}
 
                     else:
                         # just part of template
@@ -423,65 +494,101 @@ class Parser(object):
                                 _new_path += _atom
                                 _res = True
                                 break
+                            else:
+                                # HACK for * in path when more than one folder use
+                                # e.g.:
+                                # _sym = /pool/*/
+                                # _path = /pool/detects/e/filename.deb
+                                try:
+                                    if '*' in _sym:
+                                        re_str = fnmatch.translate(_sym)
+                                        # \/pool\/.*\/\Z(?ms) => \/pool\/.*\/
+                                        if re_str.endswith('\\Z(?ms)'):
+                                            re_str = re_str[:-7]
+                                        found_str = re.match(re_str, _path).group()
+                                        _path = _path[len(found_str):]
+                                        _new_path += found_str
+                                        _res = True
+                                        break
+                                except Exception as e:
+                                    logging.error("Something wrong when parse '{}' in '{}'".format(_sym, _path))
+                                    logging.exception(e)
 
                         if not _res:
-                            return False, {}
-            return _res, _res_params
+                            return False, {}, {}
+            return _res, _res_params, _res_params_raw
 
         _result = False
         _result_params = {}
+        _result_params_raw = {}
         # rule = self._rules[_rule_name]
         # for _rule in self._rules[_rule_name]:
         for i in range(len(self._rules[_rule_name])):
-            _ok, _params = do_check(i, str(path))
+            _ok, _params, _params_raw = do_check(i, str(path))
             if _ok:
                 _result = True
                 _result_params.update({k: v for k, v in _params.items() if k not in _result_params})
+                _result_params_raw.update({k: v for k, v in _params_raw.items() if k not in _result_params_raw})
                 break
-        return _result, _result_params
+        return _result, _result_params, _result_params_raw
 
     def validate(self, value, rule_name, params, return_params=False):
+
+        # Если правила для валидации не заданы - говорим что пакет нам подходит
         if rule_name not in self._rules:
             return (True, {}) if return_params else True
-            # raise CrosspmException(
-            #     CROSSPM_ERRORCODE_CONFIG_FORMAT_ERROR,
-            #     'Parser rule for [{}] not found in config.'.format(rule_name)
-            # )
         if len(self._rules[rule_name]) == 0:
             return (True, {}) if return_params else True
         if self._rules[rule_name] is None:
             return (True, {}) if return_params else True
-        _res = True
-        _res_params = {}
-        # _dirty = self._rules[rule_name].format(**params)
-        _all_dirties = self.fill_rule(rule_name, params, return_params=True)
+
+        _valid = True
+        _result_params = {}
+
+        # Все возможные вариант rule. Для properties - bad, snapshot, etc...
+        # Но содержим массив массивов
+        _all_dirties = self.fill_rule(rule_name, params, return_params=True, return_defaults=True)
+
         for _dirties in _all_dirties:
+            # _dirties - набор конкретных правил для валидации
             _res_sub = False
             _res_sub_params = {}
             for _dirt in _dirties:
+                # _dirty - Одно "возможное" правило?
                 _res_var = False
                 # TODO: Use split_with_regexp() instead
                 _dirty = [x.split(']') for x in _dirt['var'].split('[')]
                 _dirty = self.list_flatter(_dirty)
                 _variants = self.get_variants(_dirty, [])
-                if type(value) is str:
+
+                if isinstance(value, str):
                     _res_var = value in _variants
-                elif type(value) in (list, tuple):
+                elif isinstance(value, (list, tuple)):
                     _res_var = False
                     for _variant in _variants:
                         if _variant in value:
                             _res_var = True
                             break
-                elif type(value) is dict:
+                elif isinstance(value, dict):
+                    _key = ''
+                    if 'mask' in _dirt.get('default', {}):
+                        _mask = _dirt['default'].get('mask', '')
+                        if len(fnmatch.filter(value.keys(), _mask)) == 0:
+                            _key = _dirt['default'].get('key', '')
+                            value[_key] = [_dirt['default'].get('value', '')]
+
                     for _variant in _variants:
                         _tmp = [x.strip() for x in _variant.split('=')]
                         _tmp = [x if len(x) > 0 else '*' for x in _tmp]
-                        for _key in fnmatch.filter(value.keys(), _tmp[0]):
+                        _key_list = fnmatch.filter(value.keys(), _tmp[0])
+                        if len(_key_list) == 0 and '*' in _key:
+                            _key_list = [_key]
+                        for _key in _key_list:
                             if len(_tmp) > 1:
                                 _tmp_val = value[_key]
-                                if type(_tmp_val) is str:
+                                if isinstance(_tmp_val, str):
                                     _tmp_val = [_tmp_val]
-                                elif type(_tmp_val) not in [list, tuple, dict]:
+                                elif not isinstance(_tmp_val, (list, tuple, dict)):
                                     raise CrosspmException(
                                         CROSSPM_ERRORCODE_CONFIG_FORMAT_ERROR,
                                         'Parser rule for [{}] not able to process [{}] data type.'.format(
@@ -502,16 +609,16 @@ class Parser(object):
                 if _res_sub:
                     _res_sub_params = _dirt['params']
                     break
-            _res = _res and _res_sub
-            if _res:
-                _res_params.update(_res_sub_params)
-        return (_res, _res_params) if return_params else _res
+            _valid = _valid and _res_sub
+            if _valid:
+                _result_params.update(_res_sub_params)
+        return (_valid, _result_params) if return_params else _valid
 
     def iter_matched_values(self, column_name, value):
         _values = self._config.get_values(column_name)
         for _value in _values:
             if (value is None) or (self.values_match(_value, value, _values)):
-                if type(_values) is dict:
+                if isinstance(_values, dict):
                     _value = _values[_value]
                 yield _value
 
@@ -530,8 +637,8 @@ class Parser(object):
                 _sign = '=='
 
         var1, var2 = _value, value
-        if type(_values) is dict:
-            var2 = 0 if type(var1) is int else ''
+        if isinstance(_values, dict):
+            var2 = 0 if isinstance(var1, int) else ''
             for k, v in _values.items():
                 if value == v:
                     var2 = k
@@ -543,7 +650,7 @@ class Parser(object):
                 var2a = int(var2)
                 if not _sign:
                     _sign = '=='
-            except:
+            except Exception:
                 var1a = str(var1)
                 var2a = str(var2)
             var1, var2 = var1a, var2a
@@ -555,7 +662,7 @@ class Parser(object):
 
         return _match
 
-    def fill_rule(self, rule_name, params, return_params=False):
+    def fill_rule(self, rule_name, params, return_params=False, return_defaults=False):
         def fill_rule_inner(_cols, _params_inner, _pars=None):
             if _pars is None:
                 _pars = {}
@@ -578,7 +685,7 @@ class Parser(object):
                 if _valued:
                     _columns += [[_col, [x for x in self.iter_matched_values(_col, params[_col])]]]
                 else:
-                    if type(params[_col]) not in [list, tuple]:
+                    if not isinstance(params[_col], (list, tuple)):
                         _tmp = [params[_col]]
                     else:
                         _tmp = [x for x in params[_col]]
@@ -595,15 +702,29 @@ class Parser(object):
             for _par in fill_rule_inner(_columns, []):
                 _params.update(_par)
                 _var = self._rules[rule_name][z].format(**_params)
-                if return_params:
-                    _res_part += [{'var': _var, 'params': {k: v for k, v in _par.items()}}]
+                if return_params or return_defaults:
+                    _tmp_res_part = {'var': _var}
+                    if return_params:
+                        _tmp_res_part['params'] = {k: v for k, v in _par.items()}
+                    if return_defaults and rule_name in self._defaults_masked:
+                        _tmp_res_part['default'] = self._defaults_masked[rule_name][z]
+                    else:
+                        _tmp_res_part['default'] = {}
+                    _res_part += [_tmp_res_part]
                 else:
                     _res_part += [_var]
 
             if len(_res_part) == 0:
                 _var = self._rules[rule_name][z].format(**_params)
-                if return_params:
-                    _res_part += [{'var': _var, 'params': {}}]
+                if return_params or return_defaults:
+                    _tmp_res_part = {'var': _var}
+                    if return_params:
+                        _tmp_res_part['params'] = {}
+                    if return_defaults and rule_name in self._defaults_masked:
+                        _tmp_res_part['default'] = self._defaults_masked[rule_name][z]
+                    else:
+                        _tmp_res_part['default'] = {}
+                    _res_part += [_tmp_res_part]
                 else:
                     _res_part += [_var]
             _res += [_res_part]
@@ -616,13 +737,60 @@ class Parser(object):
                 result[k] = self.merge_with_mask(k, params[k])
         return result
 
+    def get_params_with_extra(self, rule_name, params):
+        """
+        Get params with extra, like 'any'
+        :param rule_name: 'path'
+        :param params: default params
+        :return: list of combination params
+        """
+        # HACK for prefer-local
+        result = []
+        extra_params = self._rules_vars_extra.get(rule_name, {})[0]
+        _tmp_params = copy.deepcopy(params)
+        _fixed_params = {}
+
+        # Save params with list type - this type not changed
+        for key, value in _tmp_params.items():
+            if isinstance(value, list):
+                _fixed_params[key] = value
+        _tmp_params = {k: v for k, v in _tmp_params.items() if k not in _fixed_params}
+
+        # extend with extra_vars - like 'any'
+        for key, value in _tmp_params.items():
+            if not isinstance(value, list) and key:
+                _tmp_params[key] = list([value])
+            if key in extra_params:
+                _tmp_params[key].extend(extra_params[key])
+
+        # get combinations
+        keys = sorted(_tmp_params)
+        combinations = itertools.product(*(_tmp_params[x] for x in keys))
+        for comb in combinations:
+            _dict = dict(zip(keys, comb))
+            _dict.update(_fixed_params)
+            result.append(_dict)
+
+        return result
+
     def get_paths(self, list_or_file_path, source):
         if 'path' not in self._rules:
             return None
         _paths = []
         for _params in self.iter_packages_params(list_or_file_path):
-            for _repo in source.args['repo']:
-                _params['server'] = source.args['server']
+            if _params['repo'] is None or _params['repo'] == '*':
+                repo_list = source.args['repo']
+            elif _params['repo'] not in source.args['repo']:
+                continue
+            else:
+                repo_list = [_params['repo']]
+
+            _params['server'] = source.args['server']
+            _sub_paths = {
+                'params': {k: v for k, v in _params.items() if k != 'repo'},
+                'paths': [],
+            }
+            for _repo in repo_list:
                 _params['repo'] = _repo
                 # _dirty = self._rules['path'].format(**_params)
                 _all_dirties = self.fill_rule('path', _params)
@@ -633,9 +801,10 @@ class Parser(object):
                         # TODO: Use split_with_regexp() instead
                         _dirty = [x.split(']') for x in _dirty.split('[')]
                         _dirty = self.list_flatter(_dirty)
-                        _paths += [{'paths': self.get_variants(_dirty, []),
-                                    'params': {k: v for k, v in _params.items()},
-                                    }]
+                        _sub_paths['paths'] += [{'paths': self.get_variants(_dirty, []),
+                                                 'repo': _repo,
+                                                 }]
+            _paths += [_sub_paths]
         return _paths
 
     def get_variants(self, dirty, paths):
@@ -656,22 +825,38 @@ class Parser(object):
                     break
         return paths
 
-    def iter_packages_params(self, list_or_file_path):
-        if type(list_or_file_path) is str:
+    def iter_packages_params(self, list_or_file_path, deps_content=None):
+        if deps_content is not None:
+            # HACK for download with --dependencies-content and existed file dependencies.txt.lock
+            list_or_file_path = deps_content
+
+        if list_or_file_path.__class__ is DependenciesContent:
+            # Даёт возможность передать сразу контент файла, а не файл
+            for i, line in enumerate(list_or_file_path.splitlines()):
+                yield self.get_package_params(i, line)
+        elif isinstance(list_or_file_path, str):
             if not os.path.exists(list_or_file_path):
                 raise CrosspmException(
                     CROSSPM_ERRORCODE_FILE_DEPS_NOT_FOUND,
                     'File not found: [{}]'.format(list_or_file_path),
                 )
 
-            with open(list_or_file_path, 'r') as f:
+            with open(list_or_file_path, 'r', encoding="utf-8-sig") as f:
                 for i, line in enumerate(f):
                     line = line.strip()
 
+                    #  if i == 0 and line.startswith(''.join(map(chr,(1087,187,1111)))):
+                    if i == 0 and line.startswith(chr(1087) + chr(187) + chr(1111)):  # TODO: why?
+                        line = line[3:]
+
                     if not line or line.startswith(('#', '[',)):
                         continue
 
                     yield self.get_package_params(i, line)
+        elif (isinstance(list_or_file_path, dict)) and ('raw' in list_or_file_path):
+            for _item in list_or_file_path['raw']:
+                _tmp_item = {k: self.parse_by_mask(k, v, False, True) for k, v in _item.items()}
+                yield _tmp_item
         else:
             for _item in list_or_file_path:
                 yield _item
@@ -692,14 +877,24 @@ class Parser(object):
                 'Nothing parsed at line {}: [{}]'.format(line_no, line.strip())
             )
 
-        _vars = self._config.complete_params(_vars)
+        update_items = self._config.complete_params(_vars, False)
+        update_vars = {k: self.parse_by_mask(k, v, False, True) for k, v in update_items.items()}
+        # Expend default params to passed params
+        try:
+            update_vars = {k: v.format(**_vars) if isinstance(v, str) else v for k, v in update_vars.items()}
+        except Exception as e:
+            pass
+            self._config._log.info(
+                "We catch exception when try update defaults Params, don't use this functional. Message:\n {}".format(
+                    repr(e)))
+        _vars.update(update_vars)
 
         return _vars
 
     def list_flatter(self, _src):
         _res = []
         for x in _src:
-            _res += self.list_flatter(x) if type(x) in [list, tuple] else [x]
+            _res += self.list_flatter(x) if isinstance(x, (list, tuple)) else [x]
         return _res
 
     @staticmethod
@@ -717,7 +912,7 @@ class Parser(object):
 
     def parse_value_template(self, value):
         # _regexp = ''
-        must_not = self.split_with_regexp('\[.*?\]', value)
+        must_not = self.split_with_regexp(r'\[.*?\]', value)
         for i, x in enumerate(must_not):
             must_not[i] = [self.split_with_regexp('{.*?}', x[0]), x[1]]
             # _atom = '(?P<_1_int>[\\w*><=]+)'
@@ -725,12 +920,45 @@ class Parser(object):
 
     @staticmethod
     def split_fixed_pattern(path):
+        """
+        Split path into fixed and masked parts
+        :param path: e.g
+        https://repo.example.com/artifactory/libs-cpp-release.snapshot/boost/1.60-pm/*.*.*/vc110/x86/win/boost.*.*.*.tar.gz
+        :return:
+            _path_fixed: https://repo.example.com/artifactory/libs-cpp-release.snapshot/boost/1.60-pm/
+            _path_pattern: *.*.*/vc110/x86/win/boost.*.*.*.tar.gz
+        """
         _first_pattern_pos = path.find('*')
         _path_separator_pos = path.rfind('/', 0, _first_pattern_pos) + 1
         _path_fixed = path[:_path_separator_pos]
         _path_pattern = path[_path_separator_pos:]
         return _path_fixed, _path_pattern
 
+    @staticmethod
+    def split_fixed_pattern_with_file_name(path):
+        """
+        Split path into fixed, masked parts and filename
+        :param path: e.g
+https://repo.example.com/artifactory/libs-cpp-release.snapshot/boost/1.60-pm/*.*.*/vc110/x86/win/boost.*.*.*.tar.gz
+        :return:
+            _path_fixed: https://repo.example.com/artifactory/libs-cpp-release.snapshot/boost/1.60-pm/
+            _path_pattern: *.*.*/vc100/x86/win
+            _file_name_pattern: boost.*.*.*.tar.gz
+        """
+        _first_pattern_pos = path.find('*')
+        _path_separator_pos = path.rfind('/', 0, _first_pattern_pos)
+        _path_fixed = path[:_path_separator_pos]
+        _path_pattern = path[_path_separator_pos + 1:]
+        _file_name_pattern_separator_pos = _path_pattern.rfind('/', 0)
+        _file_name_pattern = _path_pattern[_file_name_pattern_separator_pos + 1:]
+
+        if _path_pattern.find('*') == -1 or _file_name_pattern_separator_pos == -1:
+            _path_pattern = ""
+        else:
+            _path_pattern = _path_pattern[:_file_name_pattern_separator_pos]
+
+        return _path_fixed, _path_pattern, _file_name_pattern
+
     def filter_one(self, packages, params, params_found):
         def merge_params(path):
             _res_params = {k: v for k, v in params_found.get(path, {}).items()}
@@ -743,10 +971,10 @@ class Parser(object):
             for _atom_name in item['columns']:
                 if _atom_name in _atoms_found:
                     _rules = params[_atom_name]
-                    if type(_rules) not in [list, tuple]:
+                    if not isinstance(_rules, (list, tuple)):
                         _rules = [_rules]
                     _vars = _atoms_found[_atom_name]
-                    if type(_vars) not in [list, tuple]:
+                    if not isinstance(_vars, (list, tuple)):
                         _vars = [_vars]
                     i = -1
                     for _column in item['columns'][_atom_name]:
@@ -819,17 +1047,35 @@ class Parser(object):
 
         try:
             result = sorted_packages[self._index]
-        except:
+        except Exception:
             result = []
         return result
 
     def get_full_package_name(self, package):
-        pkg_name = package.get_name_and_path(True)
-        if self._config.no_fails:
-            param_list = [x for x in self._config.get_fails('unique', {})]
-            if self._config.name_column not in param_list:
-                param_list.insert(0, self._config.name_column)
-            params = package.get_params(param_list)
-            pkg_name = '/'.join(self.merge_with_mask(x, params[x]) for x in param_list)
+        param_list = [x for x in self._config.get_fails('unique', {})]
+        if self._config.name_column not in param_list:
+            param_list.insert(0, self._config.name_column)
+        params = package.get_params(param_list)
+        pkg_name = '/'.join(self.merge_with_mask(x, params[x]) for x in param_list)
 
         return pkg_name
+
+    def has_rule(self, rule_name):
+        res = False
+        if self._rules.get(rule_name, False):
+            res = True
+        return res
+
+    def get_params_from_properties(self, properties):
+        # Парсит свойства артефакта и выдаёт параметры
+        result = {y: properties.get(x, '') for x, y in self._usedby.get('property-parser', {}).items()}
+        return result
+
+    def get_params_from_path(self, path):
+        pattern = self._usedby.get('path-parser', None)
+        if pattern is None:
+            return {}
+        match = re.match(pattern, path)
+        if match is None:
+            return {}
+        return match.groupdict()
